<div class='swagger-ui'>
    <div class="wrapper">
        <section class="clear">
            {{flasgger_config.footer_text | safe}}
            <span style="float: right;">
                [Powered by <a target="_blank" href="https://github.com/rochacbruno/flasgger">Flasgger</a> {{ flasgger_version }}]
                <br>
            </span>
        </section>
    </div>
</div>
