<meta charset="UTF-8">
<title>{{ title }}</title>
<link href="https://fonts.googleapis.com/css?family=Open+Sans:400,700|Source+Code+Pro:300,600|Titillium+Web:400,600,700" rel="stylesheet">
<link rel="stylesheet" type="text/css" href="{{ swagger_ui_css }}">
<!-- Customize the app.config['SWAGGER']['favicon'] -->
<link rel="icon" type="image/png" href="{{ favicon }}" sizes="64x64 32x32 16x16" />
<style>
    html
    {
    box-sizing: border-box;
    overflow: -moz-scrollbars-vertical;
    overflow-y: scroll;
    }

    *,
    *:before,
    *:after
    {
    box-sizing: inherit;
    }

    body
    {
    margin:0;
    background: #fafafa;
    }
</style>
